#ifndef ZENTRY_H
#define ZENTRY_H


//! z-entry: virtual base class for spec and curve.

class CZentry {
 public:
    vector<double> z, dz;

    virtual ~CZentry() {}; // magic: makes dynamic_cast possible

    void copy_z_base( boost::shared_ptr<class CZentry> ein );
};


//! curve: parameters and fit outcome.

class CCurve: public CZentry {
 public:
    static const int mQuality = 3;
    vector<double> P;
    vector<bool>   fixed;
    double Quality[mQuality];
    
    void clear( void );
};


//! spec: x-y-dy data.

class CSpec: public CZentry {
 public:
    vector<double> x, y, dy;
    
    // modify:
    void push_xy( double _x, double _y );
    void push_xyd( double _x, double _y, double _dy );
    void clear(); 
    void remove( uint begin, uint end );
    void resize( uint n, bool with_dy );
    void purge_dy();

    // extract data:
    uint size() const;
    void intpol( const vector<double>& in_x, vector<double>& out_y ) const;
    double intpol( double x ) const {
        vector<double> xx(1), yy(1); xx[0] = x; intpol(xx,yy); return yy[0]; }
};


//! shared pointers.

typedef boost::shared_ptr<CZentry> PZentry;
typedef boost::shared_ptr<CCurve> PCurve;
typedef boost::shared_ptr<CSpec> PSpec;


//! at global scope, for use in sorting.

bool CompareZ( const PZentry& E1, const PZentry& E2 );


#endif
